// scriptgamedialoginterface.cpp
// Written by Dominque Douglas
// Copyright 2014 Dominque Douglas
#include "DXUT.h"
#include "scriptaudiointerface.h"
#include "audio.h"

enum AudioCues
{
    Cue_iAmmoBounce = 0,
    Cue_iAmmoFire,
    Cue_iDroidDestroyed,
    Cue_iDroidScan,
    Cue_iBackgroundMusic,
    Cue_iRoomRumble
};

XACTINDEX GetCueIndex(AudioCues cue)
{
	switch(cue)
	{
		case Cue_iAmmoBounce:
			return g_audioState.iAmmoBounce;
		case Cue_iAmmoFire:
			return g_audioState.iAmmoFire;
		case Cue_iDroidDestroyed:
			return g_audioState.iDroidDestroyed;
		case Cue_iDroidScan:
			return g_audioState.iDroidScan;
		case Cue_iBackgroundMusic:
			return g_audioState.iBackgroundMusic;
		case Cue_iRoomRumble:
			return g_audioState.iRoomRumble;
	}

	return 0;
}

void PlayAudioCueWrapper( AudioCues cue )
{
	PlayAudioCue(GetCueIndex(cue));
}

void Play3DAudioCueWrapper( AudioCues cue, const D3DXVECTOR3 &vPosition )
{
	Play3DAudioCue(GetCueIndex(cue), (D3DXVECTOR3 *)&vPosition);
}

int RegisterAudioInterface(asIScriptEngine *scriptengine)
{
	int result;

	// set the namespace
	result = scriptengine->SetDefaultNamespace("AUDIO"); 
	if(result < 0) return result;

	// first register our enum
	result = scriptengine->RegisterEnum("AudioCues");
	if(result < 0) return result;

	result = scriptengine->RegisterEnumValue("AudioCues", "Cue_iAmmoBounce", (int)Cue_iAmmoBounce);
	if(result < 0) return result;

    result = scriptengine->RegisterEnumValue("AudioCues", "Cue_iAmmoFire", (int)Cue_iAmmoFire);
	if(result < 0) return result;

    result = scriptengine->RegisterEnumValue("AudioCues", "Cue_iDroidDestroyed", (int)Cue_iDroidDestroyed);
	if(result < 0) return result;

	result = scriptengine->RegisterEnumValue("AudioCues", "Cue_iDroidScan", (int)Cue_iDroidScan);
	if(result < 0) return result;

    result = scriptengine->RegisterEnumValue("AudioCues", "Cue_iBackgroundMusic", (int)Cue_iBackgroundMusic);
	if(result < 0) return result;

    result = scriptengine->RegisterEnumValue("AudioCues", "Cue_iRoomRumble", (int)Cue_iRoomRumble);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void PlayAudioCue(AudioCues)", asFUNCTION(PlayAudioCueWrapper), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void Play3DAudioCue(AudioCues, const D3DXVECTOR3 &in)", asFUNCTION(Play3DAudioCueWrapper), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void SetNumDroidsForAudio(int)", asFUNCTION(SetNumDroidsForAudio), asCALL_CDECL);
	if(result < 0) return result;

	// reset back to global namespace
	result = scriptengine->SetDefaultNamespace(""); 

	return result;
}