#include "DXUT.h"
// scriptgamedialoginterface.cpp
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "scriptcamerainterface.h"

// declare the global g_Camera variable as extern here so we can use the one defined in game.cpp
extern CFirstPersonCamera  g_Camera;

static void CameraSetViewParams( D3DXVECTOR3 &pvEyePt, D3DXVECTOR3 &pvLookatPt )
{
	g_Camera.SetViewParams(&pvEyePt, &pvLookatPt);
}

static void CameraSetEnableYAxisMovement( bool bEnableYAxisMovement )
{
	g_Camera.SetEnableYAxisMovement(bEnableYAxisMovement);
}

static void CameraSetClipToBoundary( bool bClipToBoundary, D3DXVECTOR3 &pvMinBoundary, D3DXVECTOR3 &pvMaxBoundary )
{
	g_Camera.SetClipToBoundary(bClipToBoundary, &pvMinBoundary, &pvMaxBoundary );
}

static void CameraSetScalers( FLOAT fRotationScaler, FLOAT fMoveScaler )
{
	g_Camera.SetScalers(fRotationScaler, fMoveScaler );
}

static void CameraSetRotateButtons( bool bLeft, bool bMiddle, bool bRight, bool bRotateWithoutButtonDown)
{
	g_Camera.SetRotateButtons( bLeft, bMiddle, bRight, bRotateWithoutButtonDown);
}

static D3DXMATRIXA16 *CameraGetViewMatrix()
{
	// AngelScript expects D3DXMATRIXA16 but the camera returns D3DXMATRIX
	// must return handle because this is a scoped reference
	return new D3DXMATRIXA16(*g_Camera.GetViewMatrix());
}

int RegisterCameraInterface(asIScriptEngine *scriptengine)
{
	int result;

	// set the namespace
	result = scriptengine->SetDefaultNamespace("FirstPersonCamera"); 
	if(result < 0) return result;

	// register the global functions
	result = scriptengine->RegisterGlobalFunction("void SetViewParams( D3DXVECTOR3 &in, D3DXVECTOR3 &in )", asFUNCTION(CameraSetViewParams), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void SetEnableYAxisMovement( bool )", asFUNCTION(CameraSetEnableYAxisMovement), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void SetClipToBoundary( bool, D3DXVECTOR3 &in, D3DXVECTOR3 &in )", asFUNCTION(CameraSetClipToBoundary), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void SetScalers( float, float)", asFUNCTION(CameraSetScalers), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void SetRotateButtons( bool, bool, bool, bool = false)", asFUNCTION(CameraSetRotateButtons), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("D3DXMATRIXA16 @CameraGetViewMatrix()", asFUNCTION(CameraGetViewMatrix), asCALL_CDECL);
	if(result < 0) return result;

	// reset back to global namespace
	result = scriptengine->SetDefaultNamespace(""); 
	
	return result;
}





