// scriptd3dxcolor.cpp
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "DXUT.h"
#include "scriptd3dxcolor.h"

static void DestructD3DXCOLOR(void *thisPointer)
{
	((D3DXCOLOR*)thisPointer)->~D3DXCOLOR();
}

static void ConstructD3DXCOLOR(void *thisPointer)
{
	new(thisPointer) D3DXCOLOR();
}

// provide a copy constructor
static void CopyConstructD3DXCOLOR(const D3DXCOLOR &other, void *thisPointer)
{
	new(thisPointer) D3DXCOLOR(other);
}

// provide a constructor from 3 floats
static void ConstructD3DXCOLORFromFloats(float r, float g, float b, float a, void *thisPointer)
{
	new(thisPointer) D3DXCOLOR(r, g, b, a);
}

static D3DXCOLOR& AddAssignTwoColors /* operator += */ ( CONST D3DXCOLOR &vec, D3DXCOLOR &dest )
{
	dest += vec;

	return dest;
}

static D3DXCOLOR& MinusAssignTwoColors /* operator -= */ ( CONST D3DXCOLOR &vec, D3DXCOLOR &dest )
{
	dest -= vec;

	return dest;
}

static D3DXCOLOR& MulAssignWithFloat /* operator *= */( FLOAT f, D3DXCOLOR &dest )
{
	dest *= f;

	return dest;
}

static D3DXCOLOR& DivAssignWithFloat/* operator /= */ ( FLOAT f, D3DXCOLOR &dest )
{
	dest /= f;

	return dest;
}

static D3DXCOLOR AddTwoColors/* operator + */ ( CONST D3DXCOLOR &v, CONST D3DXCOLOR &v2 )
{
	return v + v2;
}

static D3DXCOLOR MinusTwoColors/* operator - */ ( CONST D3DXCOLOR &v, CONST D3DXCOLOR &v2 )
{
	return v - v2;
}

static D3DXCOLOR MulColorScalar /* operator * */ ( CONST D3DXCOLOR &v, FLOAT f)
{
	return v * f;
}

static D3DXCOLOR DivColorScalar/* operator / */ ( CONST D3DXCOLOR &v, FLOAT f)
{
	return v / f;
}

static BOOL ColorsEqual /* operator == */ ( CONST D3DXCOLOR &lval,  CONST D3DXCOLOR &rval)
{
	return lval == rval;
}

int RegisterD3DXCOLOR(asIScriptEngine *engine)
{
	int r;

	// Register the string type
	r = engine->RegisterObjectType("D3DXCOLOR", sizeof(D3DXCOLOR), asOBJ_VALUE | 
		                           asOBJ_APP_CLASS | asOBJ_APP_CLASS_CONSTRUCTOR | asOBJ_APP_CLASS_COPY_CONSTRUCTOR | asOBJ_APP_CLASS_DESTRUCTOR);
	if(r < 0) return r;

	// Register the object operator overloads
	r = engine->RegisterObjectBehaviour("D3DXCOLOR", asBEHAVE_CONSTRUCT,  "void f()",                    asFUNCTION(ConstructD3DXCOLOR), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXCOLOR", asBEHAVE_CONSTRUCT,  "void f(const D3DXCOLOR &in)", asFUNCTION(CopyConstructD3DXCOLOR), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXCOLOR", asBEHAVE_CONSTRUCT,  "void f(float, float, float, float)", asFUNCTION(ConstructD3DXCOLORFromFloats), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXCOLOR", asBEHAVE_DESTRUCT, "void f()", asFUNCTION(DestructD3DXCOLOR), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR &opAssign(const D3DXCOLOR &in)", asMETHODPR(D3DXCOLOR, operator =, (const D3DXCOLOR&), D3DXCOLOR&), asCALL_THISCALL);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR &opAddAssign(const D3DXCOLOR &in)", asFUNCTION(AddAssignTwoColors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR &opSubAssign(const D3DXCOLOR &in)", asFUNCTION(MinusAssignTwoColors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR &opMulAssign(float)", asFUNCTION(MulAssignWithFloat), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR &opDivAssign(float)", asFUNCTION(DivAssignWithFloat), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR opAdd(const D3DXCOLOR &in)", asFUNCTION(AddTwoColors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR opSub(const D3DXCOLOR &in)", asFUNCTION(MinusTwoColors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR opMul(float)", asFUNCTION(MulColorScalar), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR opMul_r(float)", asFUNCTION(MulColorScalar), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXCOLOR", "D3DXCOLOR opDiv(float)", asFUNCTION(DivColorScalar), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXCOLOR", "bool opEquals(const D3DXCOLOR &in) const", asFUNCTION(ColorsEqual), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectProperty("D3DXCOLOR", "float r", asOFFSET(D3DXCOLOR,r));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXCOLOR", "float g", asOFFSET(D3DXCOLOR,g));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXCOLOR", "float b", asOFFSET(D3DXCOLOR,b));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXCOLOR", "float a", asOFFSET(D3DXCOLOR,a));

	return r;
}