#pragma once
// scriptd3dxmathfunc.h
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "DXUT.h"
#include "scriptd3dxmathfunc.h"
#include "scriptd3dxvector3.h"
#include "scriptd3dxvector4.h"
#include "scriptD3DXMATRIXA16.h"
#include "scriptd3dxquaternion.h"

static void MatrixInverse(D3DXMATRIXA16 &out, float &determinant, const D3DXMATRIXA16 &in)
{
	D3DXMatrixInverse(&out, &determinant, &in);
}

static void Vec4Transform(D3DXVECTOR4 &out, const D3DXVECTOR4 &v, const D3DXMATRIXA16 &m)
{
	D3DXVec4Transform(&out, &v, &m);
}

static void QuaternionRotationYawPitchRoll(D3DXQUATERNION &out, float yaw, float pitch, float roll)
{
	D3DXQuaternionRotationYawPitchRoll(&out, yaw, pitch, roll);
}

static void MatrixRotationQuaternion(D3DXMATRIX &out, const D3DXQUATERNION &q)
{
	D3DXMatrixRotationQuaternion(&out, &q);
}

static void QuaternionSlerp(D3DXQUATERNION &out, const D3DXQUATERNION &q1, const D3DXQUATERNION &q2, float t)
{
	D3DXQuaternionSlerp(&out, &q1, &q2, t);
}

static int rand(int min, int max)
{
	int diff = max - min;
	return (rand() % diff) + min;
}

int RegisterD3DXMathFunctions(asIScriptEngine *scriptengine)
{
	int result;

	// register our types first
	result = RegisterD3DXVECTOR3(scriptengine);
	if(result < 0) return result;
	result = RegisterD3DXVECTOR4(scriptengine);
	if(result < 0) return result;
	result = RegisterD3DXQUATERNION(scriptengine);
	if(result < 0) return result;
	result = RegisterD3DXMATRIXA16(scriptengine);
	if(result < 0) return result;

	// register the global functions
	result = scriptengine->RegisterGlobalFunction("void D3DXMatrixInverse(D3DXMATRIXA16 &out, float &out, const D3DXMATRIXA16 &in)", asFUNCTION(MatrixInverse), asCALL_CDECL);
	if(result < 0) return result;
	result = scriptengine->RegisterGlobalFunction("void D3DXVec4Transform(D3DXVECTOR4 &out, const D3DXVECTOR4 &in, const D3DXMATRIXA16 &in)", asFUNCTION(Vec4Transform), asCALL_CDECL);
	if(result < 0) return result;
	result = scriptengine->RegisterGlobalFunction("void D3DXQuaternionRotationYawPitchRoll(D3DXQUATERNION &out, float, float, float)", asFUNCTION(QuaternionRotationYawPitchRoll), asCALL_CDECL);
	if(result < 0) return result;
	result = scriptengine->RegisterGlobalFunction("void D3DXMatrixRotationQuaternion(D3DXMATRIXA16 &out, const D3DXQUATERNION &in)", asFUNCTION(MatrixRotationQuaternion), asCALL_CDECL);
	if(result < 0) return result;
	result = scriptengine->RegisterGlobalFunction("void D3DXQuaternionSlerp(D3DXQUATERNION &out, const D3DXQUATERNION &in, const D3DXQUATERNION &in, float t)", asFUNCTION(QuaternionSlerp), asCALL_CDECL);
	if(result < 0) return result;

	// add rand() here
	result = scriptengine->RegisterGlobalFunction("int rand(int, int)", asFUNCTIONPR(rand, (int, int), int), asCALL_CDECL);

	return result;
}



