// scriptD3DXMATRIXA16.h
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "DXUT.h"
#include "scriptD3DXMATRIXA16.h"

static D3DXMATRIXA16 *D3DXMATRIXA16_Factory()
{
  return new D3DXMATRIXA16;
}

static D3DXMATRIXA16 *D3DXMATRIXA16_FactoryCopy(const D3DXMATRIXA16 &other)
{
  return new D3DXMATRIXA16(other);
}

static D3DXMATRIXA16 *D3DXMATRIXA16_FactoryFromfloats(float _11, float _12, float _13, float _14,
												float _21, float _22, float _23, float _24,
												float _31, float _32, float _33, float _34,
												float _41, float _42, float _43, float _44)
{
  return new D3DXMATRIXA16(_11, _12, _13, _14,
						   _21, _22, _23, _24,
						   _31, _32, _33, _34,
						   _41, _42, _43, _44);
}

static void D3DXMATRIXA16_Release(D3DXMATRIXA16 *s)
{
  if( s ) delete s;
}

static D3DXMATRIXA16& Matrix_opMulAssign /* operator *= */ ( const D3DXMATRIXA16 &m, D3DXMATRIXA16 &dest )
{
	dest *= m;

	return dest;
}

static D3DXMATRIXA16& Matrix_opAddAssign /* operator += */ ( const D3DXMATRIXA16 &m, D3DXMATRIXA16 &dest )
{
	dest += m;

	return dest;
}

static D3DXMATRIXA16& Matrix_opSubAssign /* operator -= */ ( const D3DXMATRIXA16 &m, D3DXMATRIXA16 &dest )
{
	dest -= m;

	return dest;
}

static D3DXMATRIXA16& Matrix_opMulAssignF /* operator *= */ ( float f, D3DXMATRIXA16 &dest )
{
	dest *= f;

	return dest;
}

static D3DXMATRIXA16& Matrix_opDivAssign /* operator /= */ ( float f, D3DXMATRIXA16 &dest )
{
	dest /= f;

	return dest;
}

static D3DXMATRIXA16 *Matrix_opMul /* operator * */ ( const D3DXMATRIXA16 &lhs, const D3DXMATRIXA16 &rhs)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(lhs * rhs);
}

static D3DXMATRIXA16 *Matrix_opAdd /* operator + */ ( const D3DXMATRIXA16 &lhs, const D3DXMATRIXA16 &rhs)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(lhs + rhs);
}

static D3DXMATRIXA16 *Matrix_opSub /* operator - */ ( const D3DXMATRIXA16 &lhs, const D3DXMATRIXA16 &rhs)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(lhs - rhs);
}

static D3DXMATRIXA16 *Matrix_opMulF /* operator * */ ( const D3DXMATRIXA16 &lhs, float rhs)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(lhs * rhs);
}

static D3DXMATRIXA16 *Matrix_opMulF_r /* operator * */ ( const D3DXMATRIXA16 &lhs, float rhs)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(rhs * lhs);
}

static D3DXMATRIXA16 *Matrix_opDiv /* operator / */ ( const D3DXMATRIXA16 &lhs, float rhs)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(lhs / rhs);
}


 // unary operators
 static D3DXMATRIXA16 *Matrix_opNeg /* operator - */ (const D3DXMATRIXA16 &m)
{
	// (From AngelScript Manual) any function that either takes or returns the type by value in C++ must
	// be wrapped in order to permit AngelScript to manage the life time of the values
	return new D3DXMATRIXA16(-m);
}

static BOOL MatricesEqual /* operator == */ ( CONST D3DXMATRIXA16 &lval,  CONST D3DXMATRIXA16 &rval)
{
	return lval == rval;
}

int RegisterD3DXMATRIXA16(asIScriptEngine *engine)
{
	int r;

	// Register the string type
	r = engine->RegisterObjectType("D3DXMATRIXA16", 0, asOBJ_REF | asOBJ_SCOPED);
	if(r < 0) return r;

	// with reference types we register factores and not constructors
	r = engine->RegisterObjectBehaviour("D3DXMATRIXA16", asBEHAVE_FACTORY, "D3DXMATRIXA16 @f()", asFUNCTION(D3DXMATRIXA16_Factory), asCALL_CDECL);
	r = engine->RegisterObjectBehaviour("D3DXMATRIXA16", asBEHAVE_FACTORY, "D3DXMATRIXA16 @f(const D3DXMATRIXA16 &in)", asFUNCTION(D3DXMATRIXA16_FactoryCopy), asCALL_CDECL);
	r = engine->RegisterObjectBehaviour("D3DXMATRIXA16", asBEHAVE_FACTORY, 
		"D3DXMATRIXA16 @f(float, float, float, float, float, float, float, float, float, float, float, float, float, float, float, float)",
		asFUNCTION(D3DXMATRIXA16_FactoryFromfloats), asCALL_CDECL);
	r = engine->RegisterObjectBehaviour("D3DXMATRIXA16", asBEHAVE_RELEASE, "void f()", asFUNCTION(D3DXMATRIXA16_Release), asCALL_CDECL_OBJLAST);

	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 &opMulAssign( const D3DXMATRIXA16 &in)", asFUNCTION(Matrix_opMulAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 &opAddAssign( const D3DXMATRIXA16 &in)", asFUNCTION(Matrix_opAddAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 &opSubAssign( const D3DXMATRIXA16 &in)", asFUNCTION(Matrix_opSubAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 &opMulAssign( float)", asFUNCTION(Matrix_opMulAssignF), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 &opDivAssign( float)", asFUNCTION(Matrix_opDivAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;

	// instead of returning a value, the operator must return a handle because this is a scoped reference type
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opMul(const D3DXMATRIXA16 &in)", asFUNCTION(Matrix_opMul), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opAdd(const D3DXMATRIXA16 &in)", asFUNCTION(Matrix_opAdd), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opSub(const D3DXMATRIXA16 &in)", asFUNCTION(Matrix_opSub), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opMul(float )", asFUNCTION(Matrix_opMulF), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opMul_r(float )", asFUNCTION(Matrix_opMulF_r), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opDiv(float )", asFUNCTION(Matrix_opDiv), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 @Matrix_opNeg()", asFUNCTION(Matrix_opNeg), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "D3DXMATRIXA16 &opAssign(const D3DXMATRIXA16 &in)", asMETHODPR(D3DXMATRIXA16, operator =, (const D3DXMATRIXA16&), D3DXMATRIXA16&), asCALL_THISCALL);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXMATRIXA16", "bool opEquals(const D3DXMATRIXA16 &in) const", asFUNCTION(MatricesEqual), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	// register the properties
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _11", asOFFSET(D3DXMATRIXA16,_11));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _12", asOFFSET(D3DXMATRIXA16,_12));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _13", asOFFSET(D3DXMATRIXA16,_13));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _14", asOFFSET(D3DXMATRIXA16,_14));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _21", asOFFSET(D3DXMATRIXA16,_21));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _22", asOFFSET(D3DXMATRIXA16,_22));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _23", asOFFSET(D3DXMATRIXA16,_23));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _24", asOFFSET(D3DXMATRIXA16,_24));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _31", asOFFSET(D3DXMATRIXA16,_31));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _32", asOFFSET(D3DXMATRIXA16,_32));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _33", asOFFSET(D3DXMATRIXA16,_33));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _34", asOFFSET(D3DXMATRIXA16,_34));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _41", asOFFSET(D3DXMATRIXA16,_41));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _42", asOFFSET(D3DXMATRIXA16,_42));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _43", asOFFSET(D3DXMATRIXA16,_43));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXMATRIXA16", "float _44", asOFFSET(D3DXMATRIXA16,_44));

	return r;
}

