// scriptd3dxquaternion.h
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "DXUT.h"
#include "scriptd3dxvector4.h"

static void DestructQuaternion(void *thisPointer)
{
	((D3DXQUATERNION*)thisPointer)->~D3DXQUATERNION();
}

static void constructQuaternion(void *thisPointer)
{
	new(thisPointer) D3DXQUATERNION();
}

// provide a copy constructor
static void CopyconstructQuaternion(const D3DXQUATERNION &other, void *thisPointer)
{
	new(thisPointer) D3DXQUATERNION(other);
}

// provide a constructor from 4 floats
static void ConstructFromfloats(float x, float y, float z, float w, void *thisPointer)
{
	new(thisPointer) D3DXQUATERNION(x, y, z, w);
}

static D3DXQUATERNION &Quaternion_opAddAssign /* operator += */ ( const D3DXQUATERNION &rhs, D3DXQUATERNION &dest )
{
	dest += rhs;

	return dest;
}

static D3DXQUATERNION &Quaternion_opSubAssign /* operator -= */ ( const D3DXQUATERNION &rhs, D3DXQUATERNION &dest )
{
	dest -= rhs;

	return dest;
}

static D3DXQUATERNION &Quaternion_opMulAssign /* operator *= */ ( const D3DXQUATERNION &rhs, D3DXQUATERNION &dest )
{
	dest *= rhs;

	return dest;
}

static D3DXQUATERNION &Quaternion_opMulAssignF /* operator *= */ ( float rhs, D3DXQUATERNION &dest)
{
	dest *= rhs;

	return dest;
}

static D3DXQUATERNION &Quaternion_opDivAssign /* operator /= */ ( float rhs, D3DXQUATERNION &dest)
{
	dest /= rhs;

	return dest;
}

// unary operators
static D3DXQUATERNION Quaternion_opNeg /* operator - */ (D3DXQUATERNION &q)
{
	return -q;
}


// binary operators
static D3DXQUATERNION Quaternion_opAdd /* operator + */ ( const D3DXQUATERNION &lhs, const D3DXQUATERNION &rhs )
{
	return lhs + rhs;
}

static D3DXQUATERNION Quaternion_opSub /* operator - */ ( const D3DXQUATERNION &lhs, const D3DXQUATERNION &rhs )
{
	return lhs - rhs;
}

static D3DXQUATERNION Quaternion_opMul /* operator * */ ( const D3DXQUATERNION &lhs, const D3DXQUATERNION &rhs )
{
	return lhs * rhs;
}

static D3DXQUATERNION Quaternion_opMulF /* operator * */ ( const D3DXQUATERNION &lhs, float rhs)
{
	return lhs * rhs;
}

static D3DXQUATERNION Quaternion_opMulF_r /* operator * */ ( float lhs, const D3DXQUATERNION &rhs )
{
	return lhs * rhs;
}

static D3DXQUATERNION Quaternion_opDiv /* operator / */ ( const D3DXQUATERNION &lhs, float rhs)
{
	return lhs / rhs;
}

static bool Quaternion_opEquals/* operator == */( const D3DXQUATERNION &lhs, const D3DXQUATERNION &rhs )
{
	return (lhs == rhs) != 0; // overloaded operator returns an int an not a bool. make a bool
}

int RegisterD3DXQUATERNION(asIScriptEngine *engine)
{
	int r;

	// Register the string type
	r = engine->RegisterObjectType("D3DXQUATERNION", sizeof(D3DXQUATERNION), asOBJ_VALUE | 
		                           asOBJ_APP_CLASS | asOBJ_APP_CLASS_CONSTRUCTOR | asOBJ_APP_CLASS_COPY_CONSTRUCTOR | asOBJ_APP_CLASS_DESTRUCTOR);
	if(r < 0) return r;

	// Register the object operator overloads
	r = engine->RegisterObjectBehaviour("D3DXQUATERNION", asBEHAVE_CONSTRUCT,  "void f()",                    asFUNCTION(constructQuaternion), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXQUATERNION", asBEHAVE_CONSTRUCT,  "void f(const D3DXQUATERNION &in)", asFUNCTION(CopyconstructQuaternion), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXQUATERNION", asBEHAVE_CONSTRUCT,  "void f(float, float, float, float)", asFUNCTION(ConstructFromfloats), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXQUATERNION", asBEHAVE_DESTRUCT, "void f()", asFUNCTION(DestructQuaternion), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION &opAssign(const D3DXQUATERNION &in)", asMETHODPR(D3DXQUATERNION, operator =, (const D3DXQUATERNION&), D3DXQUATERNION&), asCALL_THISCALL);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION &Quaternion_opAddAssign ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opAddAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION &Quaternion_opSubAssign ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opSubAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION &Quaternion_opMulAssign ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opMulAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION &Quaternion_opMulAssignF( float )", asFUNCTION(Quaternion_opMulAssignF), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION &Quaternion_opDivAssign ( float )", asFUNCTION(Quaternion_opDivAssign), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opNeg ()", asFUNCTION(Quaternion_opNeg), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opAdd ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opAdd), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opSub ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opSub), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opMul ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opMul), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opMulF ( float )", asFUNCTION(Quaternion_opMulF), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opMulF_r ( float  )", asFUNCTION(Quaternion_opMulF_r), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "D3DXQUATERNION Quaternion_opDiv ( float )", asFUNCTION(Quaternion_opDiv), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXQUATERNION", "bool Quaternion_opEquals ( const D3DXQUATERNION &in )", asFUNCTION(Quaternion_opEquals), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectProperty("D3DXQUATERNION", "float x", asOFFSET(D3DXQUATERNION,x));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXQUATERNION", "float y", asOFFSET(D3DXQUATERNION,y));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXQUATERNION", "float z", asOFFSET(D3DXQUATERNION,z));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXQUATERNION", "float w", asOFFSET(D3DXQUATERNION,w));

	return r;
}


