// scriptd3dxvector3.h
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "DXUT.h"
#include "scriptd3dxvector3.h"

static void DestructVector3(void *thisPointer)
{
	((D3DXVECTOR3*)thisPointer)->~D3DXVECTOR3();
}

static void ConstructVector3(void *thisPointer)
{
	new(thisPointer) D3DXVECTOR3();
}

// provide a copy constructor
static void CopyConstructVector3(const D3DXVECTOR3 &other, void *thisPointer)
{
	new(thisPointer) D3DXVECTOR3(other);
}

// provide a constructor from 3 floats
static void ConstructVector3FromFloats(float x, float y, float z, void *thisPointer)
{
	new(thisPointer) D3DXVECTOR3(x, y, z);
}

static D3DXVECTOR3& AddAssignTwoVectors /* operator += */ ( CONST D3DXVECTOR3 &vec, D3DXVECTOR3 &dest )
{
	dest += vec;

	return dest;
}

static D3DXVECTOR3& MinusAssignTwoVectors /* operator -= */ ( CONST D3DXVECTOR3 &vec, D3DXVECTOR3 &dest )
{
	dest -= vec;

	return dest;
}

static D3DXVECTOR3& MulAssignWithFloat /* operator *= */( FLOAT f, D3DXVECTOR3 &dest )
{
	dest *= f;

	return dest;
}

static D3DXVECTOR3& DivAssignWithFloat/* operator /= */ ( FLOAT f, D3DXVECTOR3 &dest )
{
	dest /= f;

	return dest;
}

static D3DXVECTOR3 AddTwoVectors/* operator + */ ( CONST D3DXVECTOR3 &v, CONST D3DXVECTOR3 &v2 )
{
	return v + v2;
}

static D3DXVECTOR3 MinusTwoVectors/* operator - */ ( CONST D3DXVECTOR3 &v, CONST D3DXVECTOR3 &v2 )
{
	return v - v2;
}

static D3DXVECTOR3 MulVectorScalar /* operator * */ ( CONST D3DXVECTOR3 &v, FLOAT f)
{
	return v * f;
}

static D3DXVECTOR3 DivVectorScalar/* operator / */ ( CONST D3DXVECTOR3 &v, FLOAT f)
{
	return v / f;
}

static BOOL VectorsEqual /* operator == */ ( CONST D3DXVECTOR3 &lval,  CONST D3DXVECTOR3 &rval)
{
	return lval == rval;
}

int RegisterD3DXVECTOR3(asIScriptEngine *engine)
{
	int r;

	// Register the string type
	r = engine->RegisterObjectType("D3DXVECTOR3", sizeof(D3DXVECTOR3), asOBJ_VALUE | 
		                           asOBJ_APP_CLASS | asOBJ_APP_CLASS_CONSTRUCTOR | asOBJ_APP_CLASS_COPY_CONSTRUCTOR | asOBJ_APP_CLASS_DESTRUCTOR);
	if(r < 0) return r;

	// Register the object operator overloads
	r = engine->RegisterObjectBehaviour("D3DXVECTOR3", asBEHAVE_CONSTRUCT,  "void f()",                    asFUNCTION(ConstructVector3), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXVECTOR3", asBEHAVE_CONSTRUCT,  "void f(const D3DXVECTOR3 &in)", asFUNCTION(CopyConstructVector3), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXVECTOR3", asBEHAVE_CONSTRUCT,  "void f(float, float, float)", asFUNCTION(ConstructVector3FromFloats), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXVECTOR3", asBEHAVE_DESTRUCT, "void f()", asFUNCTION(DestructVector3), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 &opAssign(const D3DXVECTOR3 &in)", asMETHODPR(D3DXVECTOR3, operator =, (const D3DXVECTOR3&), D3DXVECTOR3&), asCALL_THISCALL);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 &opAddAssign(const D3DXVECTOR3 &in)", asFUNCTION(AddAssignTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 &opSubAssign(const D3DXVECTOR3 &in)", asFUNCTION(MinusAssignTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 &opMulAssign(float)", asFUNCTION(MulAssignWithFloat), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 &opDivAssign(float)", asFUNCTION(DivAssignWithFloat), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 opAdd(const D3DXVECTOR3 &in)", asFUNCTION(AddTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 opSub(const D3DXVECTOR3 &in)", asFUNCTION(MinusTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 opMul(float)", asFUNCTION(MulVectorScalar), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 opMul_r(float)", asFUNCTION(MulVectorScalar), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR3", "D3DXVECTOR3 opDiv(float)", asFUNCTION(DivVectorScalar), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXVECTOR3", "bool opEquals(const D3DXVECTOR3 &in) const", asFUNCTION(VectorsEqual), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectProperty("D3DXVECTOR3", "float x", asOFFSET(D3DXVECTOR3,x));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXVECTOR3", "float y", asOFFSET(D3DXVECTOR3,y));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXVECTOR3", "float z", asOFFSET(D3DXVECTOR3,z));

	return r;
}