// scriptd3dxvector4.h
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas
#include "DXUT.h"
#include "scriptd3dxvector4.h"

static void DestructVector4(void *thisPointer)
{
	((D3DXVECTOR4*)thisPointer)->~D3DXVECTOR4();
}

static void ConstructVector4(void *thisPointer)
{
	new(thisPointer) D3DXVECTOR4();
}

// provide a copy constructor
static void CopyConstructVector4(const D3DXVECTOR4 &other, void *thisPointer)
{
	new(thisPointer) D3DXVECTOR4(other);
}

// provide a constructor from 4 floats
static void ConstructVector4Fromfloats(float x, float y, float z, float w, void *thisPointer)
{
	new(thisPointer) D3DXVECTOR4(x, y, z, w);
}

static D3DXVECTOR4& AddAssignTwoVectors /* operator += */ ( const D3DXVECTOR4 &vec, D3DXVECTOR4 &dest )
{
	dest += vec;

	return dest;
}

static D3DXVECTOR4& MinusAssignTwoVectors /* operator -= */ ( const D3DXVECTOR4 &vec, D3DXVECTOR4 &dest )
{
	dest -= vec;

	return dest;
}

static D3DXVECTOR4& MulAssignWithfloat /* operator *= */( float f, D3DXVECTOR4 &dest )
{
	dest *= f;

	return dest;
}

static D3DXVECTOR4& DivAssignWithfloat/* operator /= */ ( float f, D3DXVECTOR4 &dest )
{
	dest /= f;

	return dest;
}

static D3DXVECTOR4 AddTwoVectors/* operator + */ ( const D3DXVECTOR4 &v, const D3DXVECTOR4 &v2 )
{
	return v + v2;
}

static D3DXVECTOR4 MinusTwoVectors/* operator - */ ( const D3DXVECTOR4 &v, const D3DXVECTOR4 &v2 )
{
	return v - v2;
}

static D3DXVECTOR4 MulVectorScalar /* operator * */ ( const D3DXVECTOR4 &v, float f)
{
	return v * f;
}

static D3DXVECTOR4 DivVectorScalar/* operator / */ ( const D3DXVECTOR4 &v, float f)
{
	return v / f;
}

static bool VectorsEqual /* operator == */ ( const D3DXVECTOR4 &lval,  const D3DXVECTOR4 &rval)
{
	return (lval == rval) != 0;  // overloaded operator returns an int an not a bool. make a bool
}

int RegisterD3DXVECTOR4(asIScriptEngine *engine)
{
	int r;

	// Register the string type
	r = engine->RegisterObjectType("D3DXVECTOR4", sizeof(D3DXVECTOR4), asOBJ_VALUE | 
		                           asOBJ_APP_CLASS | asOBJ_APP_CLASS_CONSTRUCTOR | asOBJ_APP_CLASS_COPY_CONSTRUCTOR | asOBJ_APP_CLASS_DESTRUCTOR);
	if(r < 0) return r;

	// Register the object operator overloads
	r = engine->RegisterObjectBehaviour("D3DXVECTOR4", asBEHAVE_CONSTRUCT,  "void f()",                    asFUNCTION(ConstructVector4), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXVECTOR4", asBEHAVE_CONSTRUCT,  "void f(const D3DXVECTOR4 &in)", asFUNCTION(CopyConstructVector4), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXVECTOR4", asBEHAVE_CONSTRUCT,  "void f(float, float, float, float)", asFUNCTION(ConstructVector4Fromfloats), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectBehaviour("D3DXVECTOR4", asBEHAVE_DESTRUCT, "void f()", asFUNCTION(DestructVector4), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 &opAssign(const D3DXVECTOR4 &in)", asMETHODPR(D3DXVECTOR4, operator =, (const D3DXVECTOR4&), D3DXVECTOR4&), asCALL_THISCALL);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 &opAddAssign(const D3DXVECTOR4 &in)", asFUNCTION(AddAssignTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 &opSubAssign(const D3DXVECTOR4 &in)", asFUNCTION(MinusAssignTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 &opMulAssign(float)", asFUNCTION(MulAssignWithfloat), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 &opDivAssign(float)", asFUNCTION(DivAssignWithfloat), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 opAdd(const D3DXVECTOR4 &in)", asFUNCTION(AddTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 opSub(const D3DXVECTOR4 &in)", asFUNCTION(MinusTwoVectors), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 opMul(float)", asFUNCTION(MulVectorScalar), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 opMul_r(float)", asFUNCTION(MulVectorScalar), asCALL_CDECL_OBJLAST);
	if(r < 0) return r;
	r = engine->RegisterObjectMethod("D3DXVECTOR4", "D3DXVECTOR4 opDiv(float)", asFUNCTION(DivVectorScalar), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectMethod("D3DXVECTOR4", "bool opEquals(const D3DXVECTOR4 &in) const", asFUNCTION(VectorsEqual), asCALL_CDECL_OBJFIRST);
	if(r < 0) return r;

	r = engine->RegisterObjectProperty("D3DXVECTOR4", "float x", asOFFSET(D3DXVECTOR4,x));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXVECTOR4", "float y", asOFFSET(D3DXVECTOR4,y));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXVECTOR4", "float z", asOFFSET(D3DXVECTOR4,z));
	if(r < 0) return r;
	r = engine->RegisterObjectProperty("D3DXVECTOR4", "float w", asOFFSET(D3DXVECTOR4,w));

	return r;
}