#pragma once
// scriptgamedialoginterface.cpp
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas

#include "DXUT.h"
#include <string>
#include "scriptgamedialoginterface.h"

// declare the global RENDER_STATE variable as extern here so we can use the one defined in game.cpp
extern RENDER_STATE g_Render;

static void AddStaticToDialog(XACTGAMEDIALOG dialogID, int ID, const std::string &strText, int x, int y, int width, int height, bool bIsDefault)
{
	// first get a pointer to the dialog we will use
	CDXUTDialog *the_dialog;
	if(dialogID == IDMainMenuDlg)		the_dialog = &g_Render.MainMenuDlg;
	else if(dialogID == IDVideoMenuDlg)	the_dialog = &g_Render.VideoMenuDlg;
	else if(dialogID == IDAudioMenuDlg)	the_dialog = &g_Render.AudioMenuDlg;
	else return;

	// DXUT can only handle wstrings so we need to convert
	// this only works for ASCII strings and won't convert UTF-8
	std::wstring text(strText.begin(), strText.end());

	// call the real method
	the_dialog->AddStatic(ID, text.c_str(), x, y, width, height, bIsDefault);
}

static void AddButtonToDialog(XACTGAMEDIALOG dialogID, int ID, const std::string &strText, int x, int y, int width, int height, UINT nHotkey, bool bIsDefault)
{
	// first get a pointer to the dialog we will use
	CDXUTDialog *the_dialog;
	if(dialogID == IDMainMenuDlg)		the_dialog = &g_Render.MainMenuDlg;
	else if(dialogID == IDVideoMenuDlg)	the_dialog = &g_Render.VideoMenuDlg;
	else if(dialogID == IDAudioMenuDlg)	the_dialog = &g_Render.AudioMenuDlg;
	else return;

	// DXUT can only handle wstrings so we need to convert
	// this only works for ASCII strings and won't convert UTF-8
	std::wstring text(strText.begin(), strText.end());

	// call the real method
	the_dialog->AddButton(ID, text.c_str(), x, y, width, height, nHotkey, bIsDefault);
}

static void AddCheckBoxToDialog(XACTGAMEDIALOG dialogID, int ID, const std::string &strText, int x, int y, int width,	int height, bool bChecked, UINT nHotkey, bool bIsDefault)
{
	// first get a pointer to the dialog we will use
	CDXUTDialog *the_dialog;
	if(dialogID == IDMainMenuDlg)		the_dialog = &g_Render.MainMenuDlg;
	else if(dialogID == IDVideoMenuDlg)	the_dialog = &g_Render.VideoMenuDlg;
	else if(dialogID == IDAudioMenuDlg)	the_dialog = &g_Render.AudioMenuDlg;
	else return;

	// DXUT can only handle wstrings so we need to convert
	// this only works for ASCII strings and won't convert UTF-8
	std::wstring text(strText.begin(), strText.end());

	// call the real method
	the_dialog->AddCheckBox(ID, text.c_str(), x, y, width, height, bChecked, nHotkey, bIsDefault);
}

static void AddRadioButtonToDialog(XACTGAMEDIALOG dialogID, int ID, UINT nButtonGroup, const std::string &strText, int x, int y, int width, int height, bool bChecked, UINT nHotkey, bool bIsDefault)
{
	// first get a pointer to the dialog we will use
	CDXUTDialog *the_dialog;
	if(dialogID == IDMainMenuDlg)		the_dialog = &g_Render.MainMenuDlg;
	else if(dialogID == IDVideoMenuDlg)	the_dialog = &g_Render.VideoMenuDlg;
	else if(dialogID == IDAudioMenuDlg)	the_dialog = &g_Render.AudioMenuDlg;
	else return;

	// DXUT can only handle wstrings so we need to convert
	// this only works for ASCII strings and won't convert UTF-8
	std::wstring text(strText.begin(), strText.end());

	// call the real method
	the_dialog->AddRadioButton(ID, nButtonGroup, text.c_str(), x, y, width, height, bChecked, nHotkey, bIsDefault);
}

static void AddComboBoxToDialog(XACTGAMEDIALOG dialogID, int ID, int x, int y, int width, int height,	UINT nHotKey, bool bIsDefault)
{
	// first get a pointer to the dialog we will use
	CDXUTDialog *the_dialog;
	if(dialogID == IDMainMenuDlg)		the_dialog = &g_Render.MainMenuDlg;
	else if(dialogID == IDVideoMenuDlg)	the_dialog = &g_Render.VideoMenuDlg;
	else if(dialogID == IDAudioMenuDlg)	the_dialog = &g_Render.AudioMenuDlg;
	else return;

	// call the real method
	the_dialog->AddComboBox(ID, x, y, width, height, nHotKey, bIsDefault);
}

static void AddSliderToDialog(XACTGAMEDIALOG dialogID, int ID, int x, int y, int width, int height, int min, int max, int value,  bool bIsDefault)
{
	// first get a pointer to the dialog we will use
	CDXUTDialog *the_dialog;
	if(dialogID == IDMainMenuDlg)		the_dialog = &g_Render.MainMenuDlg;
	else if(dialogID == IDVideoMenuDlg)	the_dialog = &g_Render.VideoMenuDlg;
	else if(dialogID == IDAudioMenuDlg)	the_dialog = &g_Render.AudioMenuDlg;
	else return;

	// call the real method
	the_dialog->AddSlider(ID, x, y, width, height, min, max, value, bIsDefault);
}

int RegisterDialogInterface(asIScriptEngine *scriptengine)
{
	int result;

	// set the namespace
	result = scriptengine->SetDefaultNamespace("dialogs"); 
	if(result < 0) return result;

	// first register our enum
	result = scriptengine->RegisterEnum("XACTGAMEDIALOG");
	if(result < 0) return result;

	result = scriptengine->RegisterEnumValue("XACTGAMEDIALOG", "IDMainMenuDlg", (int)IDMainMenuDlg);
	if(result < 0) return result;

    result = scriptengine->RegisterEnumValue("XACTGAMEDIALOG", "IDVideoMenuDlg", (int)IDVideoMenuDlg);
	if(result < 0) return result;

    result = scriptengine->RegisterEnumValue("XACTGAMEDIALOG", "IDAudioMenuDlg", (int)IDAudioMenuDlg);
	if(result < 0) return result;

	// register the global functions
	result = scriptengine->RegisterGlobalFunction("void AddStaticToDialog(XACTGAMEDIALOG, int, const string &in, int, int, int, int, bool = false)",
		asFUNCTION(AddStaticToDialog), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void AddButtonToDialog(XACTGAMEDIALOG, int, const string &in, int, int, int, int, uint = 0, bool = false)",
		asFUNCTION(AddButtonToDialog), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void AddCheckBoxToDialog(XACTGAMEDIALOG, int, const string &in, int, int, int, int, bool = false, uint = 0, bool = false)",
		asFUNCTION(AddCheckBoxToDialog), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void AddRadioButtonToDialog(XACTGAMEDIALOG, int, uint, const string &in, int, int, int, int, bool = false, uint = 0, bool = false)",
		asFUNCTION(AddRadioButtonToDialog), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void AddComboBoxToDialog(XACTGAMEDIALOG, int, int, int, int, int, uint = 0, bool = false)",
		asFUNCTION(AddComboBoxToDialog), asCALL_CDECL);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void AddSliderToDialog(XACTGAMEDIALOG, int, int, int, int, int, int = 0, int = 100, int = 50, bool = false)",
		asFUNCTION(AddSliderToDialog), asCALL_CDECL);
	if(result < 0) return result;

	// reset back to global namespace
	result = scriptengine->SetDefaultNamespace(""); 
	
	return result;
}
