#include "DXUT.h"
#include "scriptgamestate.h"

// declare the global g_GameState variable as extern here so we can use the one defined in game.cpp
extern GAME_STATE g_GameState;

int RegisterGameStateInterface(asIScriptEngine *scriptengine)
{
	int result;

	// set the namespace
	result = scriptengine->SetDefaultNamespace("GAME_STATE"); 
	if(result < 0) return result;

	// Register a primitive property that can be read and written to from the script.
	result = scriptengine->RegisterGlobalProperty("int nAmmoCount", &g_GameState.nAmmoCount);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalProperty("float fAmmoColorLerp", &g_GameState.fAmmoColorLerp);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalProperty("D3DXCOLOR BlendFromColor", &g_GameState.BlendFromColor);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalProperty("bool bDroidMove", &g_GameState.bDroidMove);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalProperty("bool bAutoAddDroids", &g_GameState.bAutoAddDroids);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalProperty("GAME_MODE gameMode", &g_GameState.gameMode);
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalFunction("void Reset()", asMETHOD(GAME_STATE, Reset), asCALL_THISCALL_ASGLOBAL, &g_GameState);
	if(result < 0) return result;

	// initialize the droid state and ammo state arrays
	result = g_GameState.DroidQ.InitArray(scriptengine, "array<DROID_STATE>");
	if(result < 0) return result;

	result = g_GameState.AmmoQ.InitArray(scriptengine, "array<AMMO_STATE>");
	if(result < 0) return result;
	
	// register the droid state and ammo state arrays
	result = scriptengine->RegisterGlobalProperty("array<DROID_STATE> DroidQ", g_GameState.DroidQ.GetRef());
	if(result < 0) return result;

	result = scriptengine->RegisterGlobalProperty("array<AMMO_STATE> AmmoQ", g_GameState.AmmoQ.GetRef());
	if(result < 0) return result;

	// reset back to global namespace
	result = scriptengine->SetDefaultNamespace(""); 

	return result;
}